-------------------------------------------------------------------------------
-- 3PointScale.ms
-- By Sergo Pogosyan (sergo.pogosyan@gmail.com)
-- v 1.1.1
-- Created On: 03-04-2006
-- tested using Max 7.0, 8.0
-------------------------------------------------------------------------------
-- Description: 
-- script for scaling separate objects by specifing 3 points. Like AutoCAD's 'scale by reference' command
-------------------------------------------------------------------------------
-- Usage:
-- Click 3PointScale button on the toolbar, script will ask you to enter base point.
-- Base point is the point about which scaling will performed. Second point is original point,
-- this is the point which you want to move to the new location by scaling this object.
-- And last point is reference point. Original point will jump at the location of reference
-- point when you specify them all.
-- 3PointScale script behavior is very familiar to the AutoCAD users - it is the analog
-- to the 'scale by reference' operation
-- Points have to be on one line, overwise you could get undesirable effect
-- !for correct result 2.5d snap must be chosen!
-------------------------------------------------------------------------------
-- TO DO list:
-- Major features
-- 1. 	make it to work in current coordsys
-- 		use "in coordsys (toolMode.coordSysNode) move $ (point3 0 0 1000)" to work in object's coordsys
-- 		use getRefCoordSys() for determining which coordsys is current
-- 

-- Minor features
-- 1.

-------------------------------------------------------------------------------
-- history
-- 2007-04-27: Add non-uniform scale option
-- 2007-04-28: Added working in different coordinate systems (world, scene node, local)
-- 2007-08-28: Bug fixed - in checking for good point content

scaleFactor = 1
scaleObjects =  #()
basePoint = #escape
	
on execute do
(
	rollout scaleAxis "3Point Scale"
	(
		local xAxisFlag = false
		local yAxisFlag = false
		local zAxisFlag = false
		
		fn do_scale axe1 axe2 axe3 =
		(
			if  getRefCoordSys() == #local then
			(
				scaleAxis.controls[1].caption = "Axis (in local space)"
				in coordsys local about basePoint scale scaleObjects [axe1, axe2, axe3]
			)
			else if ((getRefCoordSys() == #object) and (toolMode.coordSysNode != undefined)) then
			(
				scaleAxis.controls[1].caption = "Axis (in object space)"
				in coordsys (toolMode.coordSysNode) about basePoint scale scaleObjects [axe1, axe2, axe3]
			)
			else 
			(
				scaleAxis.controls[1].caption = "Axis (in world space)"
				about basePoint scale scaleObjects [axe1, axe2, axe3]
			)
		)
		
		group "Axis (in world space)"
		(
			checkbox xAxis "X" align:#left across:3
			checkbox yAxis "Y" align:#left across:2
			checkbox zAxis "Z" align:#left across:1
			checkbox allAxis "Uniform" align:#left
			button cancel_but "Cancel" width:60 across:2
			button ok_but "Ok"	width:60 across:1
		)
		
		on scaleAxis open do
		(
			do_scale scaleFactor scaleFactor scaleFactor
			allAxis.state = xAxis.state = yAxis.state = zAxis.state = true
		)
		
		on xAxis changed val do
		(
			if xAxis.checked then
				do_scale scaleFactor 1 1
			else
			(
				if allAxis.checked then allAxis.state = false
				do_scale (1/scaleFactor) 1 1
			)
		)
		
		on yAxis changed val do
		(
			if yAxis.checked then
				do_scale 1 scaleFactor 1
			else
			(
				if allAxis.checked then allAxis.state = false
				do_scale 1 (1/scaleFactor) 1
			)
		)
		
		on zAxis changed val do
		(
			if zAxis.checked then
				do_scale 1 1 scaleFactor
			else
			(
				if allAxis.checked then allAxis.state = false
				do_scale 1 1 (1/scaleFactor)
			)
		)
		
		on allAxis changed val do
		(	
			
			
			if allAxis.checked then				
			(
				xAxisFlag = xAxis.checked
				yAxisFlag = yAxis.checked
				zAxisFlag = zAxis.checked
				
				do_scale (if xAxisFlag then 1 else scaleFactor) (if yAxisFlag then 1 else scaleFactor) (if zAxisFlag then 1 else scaleFactor)
				xAxis.checked = yAxis.checked = zAxis.checked = true
			)
			else
			(
				do_scale (if xAxisFlag then 1 else 1/scaleFactor) (if yAxisFlag then 1 else 1/scaleFactor) (if zAxisFlag then 1 else 1/scaleFactor)
				xAxis.state = xAxisFlag
				yAxis.state = yAxisFlag
				zAxis.state = zAxisFlag
			)
		)
		
		on cancel_but pressed do
		(
			do_scale (if xAxis.checked then 1/scaleFactor else 1) (if yAxis.checked then 1/scaleFactor else 1) (if zAxis.checked then 1/scaleFactor else 1)
			destroyDialog scaleAxis --and close dialogbox
		)
		
		on ok_but pressed do
		(
			destroyDialog scaleAxis
		)
		
	)	

	--include "$scripts/SergoScripts/functions.ms"
	
	--vars
	originalPoint = #escape
	referencePoint = #escape
	
	--fill in point information
	
	print 
	basePoint = pickpoint snap:#3d prompt:"select base point"
	print basePoint
	if basePoint != #rightClick and basePoint != #escape then
	(
		originalPoint = pickpoint snap:#3d prompt:"select original point"
		print originalPoint
		if originalPoint != #rightClick and originalPoint != #escape then
		(
			referencePoint = pickpoint snap:#3d prompt:"select reference point"
			print referencePoint
			if referencePoint != #rightClick and referencePoint != #escape and referencePoint != undefined then
			(
				--scale factor calculation
				originalLength = length (basePoint - originalPoint)
				referenceLength = length (referencePoint - basePoint)
				scaleFactor = referenceLength / originalLength
		
				--actual scaling
				scaleObjects = selection as array
				scaleObjects = sergo_excludeChildren scaleObjects
				createDialog scaleAxis modal:false escapeEnable:false
			)
		)
	)	
)